% This mscript estimates cake functions as in Merlo and Tang (2010)

function [c_hat,beta_hat,diff_hat,TransMat,Rej,SampRej,bdflag,b_bdflag,table_output] ...
                = BS_EstCake(Cake,States,agree,next_States,id,flag,Rej);

% INPUTS:
% Cake   : constant vectors of censored cakes
% States : series of realized states
% id     : vectors of id's -- # of duplicate id's tells the length of
% bargaining games
% flag   : == 1 if Rej is given as an input and 0 otherwise
% if Rej is not given in the input then need to make a decision using the
% other inputs Cake,States,id ...
% Rej    : a vector of dummies showing whether the null of high agreement
% probability is rejected for a given state. (!!! Rej = 1 if the null is
% NOT rejected !!!)

% OUTPUTS: estimates for cake, discount factor, transition matrix and 
% bdflag : ==1 if p(x)<1/2 and p(x')<1/2 and upper bound reported
nx = length(id);
% -- estimate lambda_hat 
lambda_hat = zeros(4,1);
for i = 1:4,     
    lambda_hat(i) = min(nonzeros(Cake.*(States==i).*agree));  
% %     switch (sum(abs(Cake.*(States==i).*agree))==0),
% %         case 0,
% %             lambda_hat(i) = min(nonzeros(Cake.*(States==i).*agree));  
% %         case 1,
% %             lambda_hat(i) = 0;
% %     end;
end;
% -- estimate m-hat
m_hat = zeros(4,1);
for i = 1:4,
    m_hat(i) = mean(nonzeros((States==i).*(agree.*Cake+(1-agree)*lambda_hat(i)))); 
% %     if isnan(m_hat(i)), % this can happen if there is no agreement for certain states
% %         m_hat(i)=0;
% %     end;
end;
% -- estimate transition matrices
TransMat = zeros(4,4);
for i = 1:4,
    for j = 1:4,
       TransMat(i,j) = sum(nonzeros((States==i).*(next_States==j)))/sum(nonzeros((States==i)));    
    end;
    TransMat = TransMat./repmat(sum(TransMat,2),1,4);
end;


% Step 1: Estimate the discount factor 
% -- Recall beta is over-identified. Hence we will estimate beta for each x
% and then take the average
beta_hat = zeros(4,1);  %beta_hat_stk_int = zeros(4,1);
for i = 1:4,
    beta_hat(i) = lambda_hat(i)/(TransMat(i,:)*m_hat);   
end;
b_bdflag = (sign(beta_hat)+sign(beta_hat-1))*.5;
beta_hat = mean(max(0,min(beta_hat,1)));

% Step 2: Estimate c(x) 
% Preliminary Step: Estimate the agreement probabilities and test the null that p(agree) > = .5
% Rej_.. = 1 if the null is NOT rejected
[table_output,chi,pval,labels] =  crosstab(States,agree);
SampRej = (table_output(:,2) > table_output(:,1)); 
if ~flag,   
    Rej = 1-(table_output(:,2)./sum(table_output,2) <.5-1.64*sqrt((ones(4,1)*.25)./sum(table_output,2)));
end;
SampAP = table_output(:,2)/sum(table_output,2);
% Find the States for which the null is NOT rejected. 
[rind,cind,non0_vec] = find(Rej); 
% rind gives indices of states for which the agreement probability is high

% Step 2.1: Estimate c(x) for x s.t. the null p(d = 1|x) >= 1/2 is not rejected
c_hat = zeros(4,1); bdflag = zeros(4,1);
IMP_Cake = Cake.*agree; 
if ~isempty(rind),
        for i = rind',
            ind_i = nonzeros([1:1:nx]'.*(States == i)); 
            c_hat(i,1) = max(prctile(IMP_Cake(ind_i),50) , lambda_hat(i)); 
        % % %     c_hat(rind(i),1) = max( ...
        % % %           prctile(IMP_Cake(ind_i),50) , ...
        % % %           min(nonzeros((States == rind(i)).*IMP_Cake)) ...
        % % %           );
        end;
        % find the States for which the null p(d=1|x) >= 1/2 is rejected
        lp_ind = find(min(abs(repmat([1:1:4]',1,length(rind))-repmat(rind',4,1)),[],2),4);
else    c_hat = lambda_hat; bdflag = ones(4,1); 
        return; % This is the special case where all coordinates have low prob
end;

% Step 2.2: Estimate c(x) for x s.t. p(d=1|x) < 1/2

% estimation using Proposition 3
for i = lp_ind', 
    % find the candidate x' such that distribution of epsilon given x is
    % the same as given x'
    switch i,
        case 1, j = 2;
        case 2, j = 1;
        case 3, j = 4;
        case 4, j = 3;
    end;
    % Then check whether x' is such that p(agree|x')>1/2
    switch (min(abs(j-rind)) == 0),
        case 1, 
        % If yes, then go ahead and use Proposition 3 in Merlo and Tang (2011)        
        ind_i_lp = nonzeros([1:1:nx]'.*(States == i));
        Y_given_x_lp = nonzeros(IMP_Cake(ind_i_lp));    
        ind_i_hp = nonzeros([1:1:nx]'.*(States == j));
        Y_given_x_hp = nonzeros(IMP_Cake(ind_i_hp));  
        est = [];
        for alpha = [10:5:90], 
            % if prctile(Y_given_x_lp,alpha) > 0, 
            alpha_h = 1-(1-alpha)*SampAP(i)/SampAP(j);
            % % % the min operator restricts cake sizes to be larger than zero
            est = [  est ; ...
                     min(lambda_hat(i) , ...
                     prctile(Y_given_x_lp,alpha)- ...
                     min(prctile(Y_given_x_hp,alpha_h)-c_hat(j) , c_hat(j))...
                     )...
                     ];
            % % % we have used the fact that for x with lp, c(x)<=b*pi(x); and 
            % % % that eps(x)<=c(x) due to the non-negativity assumption   
            % end;
        end;     
        if ~isempty(est), 
             c_hat(i) = mean(est); 
        else c_hat(i) = lambda_hat(i); 
        end;
        if c_hat(i) == lambda_hat(i), bdflag(i) = 1; end;
        case 0, 
        % Otherwise just get an upperbound
        c_hat(i) = lambda_hat(i);
        bdflag(i) = 1;       
    end;
    
end;
for i = 1:4,
    if c_hat(i) < 0,
       c_hat(i) = 0; bdflag(i) = -1000000;
    end;
    
% % %     if c_hat(i) == lambda_hat(i);
% % %         bdflag(i) = 1000;
% % %     end;
    
end;
diff_hat = [c_hat(1)-c_hat(3) ; c_hat(2)-c_hat(4); c_hat(1)-c_hat(2); c_hat(3)-c_hat(4)];


