% This m-script performs the estimation in Merlo and Tang (2010)

clear; clc; close all;

% load /Users/xuntang/Dropbox/Antonio/Bargaining/Program/BG_Data_Oct2011.mat; 
load BG_Data_Oct2011.mat; 

S = 77; % This is simply the sample size : N = 77

% Now report: Descriptive Statistics
TempTable = tabulate(LQval); 
T_mean = mean(TempTable(:,1)); T_std = std(TempTable(:,1)); 
T_median = prctile(TempTable(:,1),50);
disp('Descriptive Data for Liquidation values: Mean, Std, Median')
disp([T_mean T_std T_median]);

TempTable = tabulate(tcl_cla); 
T_mean = mean(TempTable(:,1)); T_std = std(TempTable(:,1)); 
T_median = prctile(TempTable(:,1),50);
disp('Descriptive Data for Total Claim of Assets: Mean, Std, Median')
disp([T_mean T_std T_median]);

TempTable = tabulate(Cake); 
T_mean = mean(TempTable(:,1)); T_std = std(TempTable(:,1)); 
T_median = prctile(TempTable(:,1),50);
disp('Descriptive Data for Normalized Cake Sizes: Mean, Std, Median')
disp([T_mean T_std T_median]);

pts = 1; ido = 0; BGlength = []; 
for i = 1:length(id),
if ido == id(i), pts = pts+1; 
else BGlength = [BGlength; pts]; ido = id(i); pts = 1; 
end;
end;
T_mean = mean(BGlength); T_std = std(BGlength); 
T_median = prctile(BGlength,50);
disp('Descriptive Data for Time to agreement: Mean, Std, Median')
disp([T_mean T_std T_median]*180);

disp('****************************************************************');

[c_hat,b_hat,TM_hat,Rej,SampRej,bdflag,b_bdflag,table_output,agree] = EstCakeNew(Cake,States,id,0,[]);

% Now report: Test results for Null : p(x)>=1/2
zstat = (table_output(:,2)./sum(table_output,2)-0.5)./sqrt((ones(4,1)*.25)./sum(table_output,2));
disp('Test results in Table II: [States p_hat  p_value]');
disp([[1:4]',table_output(:,2)./sum(table_output,2),normcdf(zstat)]);
disp('****************************************************************');

% Next, report point estimates
disp('Estimation results in Table III and IV');
disp('Point Estimates for parameters : ');
c_hat
b_hat
% TM_hat
disp('Flags claculated from the estimated data : [Rej Sample_Rej Bound_Flag]')
disp([Rej SampRej bdflag b_bdflag]);
disp('****************************************************************');

% Next, report confidence intervals

TrunCake = {};
for i = 1:4, 
    % Truncated cakes 
    temp = nonzeros(Cake.*(States==i).*agree); 
    temp = sort(temp,1,'descend'); % temp = temp(3:end);
    TrunCake = {TrunCake{:},temp}; 
end;

T = 500;
BS_c_hat = zeros(4,T); BS_b_hat = zeros(1,T); BS_diff_hat = zeros(4,T);
BS_TM_hat = zeros(4,4,T); BS_flag_mat = zeros(4,4,T);
% Start doing T bootstrap simulations
t = 1; tic;
while t < T+1, % This determines the # of bootstrap iterations
    BS_States = []; BS_id = []; BS_Cake = [];
    BS_Ind = randsample([1:5,7:12,14,15,17,18,20:45,47:81,83:85],S,'true');
    % get the t-th bootstrap sample
    for s = 1:S,
        temp_ind = find((id == BS_Ind(s))); 
        temp_state = States(temp_ind); 
        BS_Cake = [BS_Cake ; Cake(temp_ind)];
        BS_States = [BS_States ; temp_state];
        BS_id = [BS_id ; id(temp_ind)];
    end;
    
    % Screen the data and only keep those with reasonable cross-tables of
    % agree and states
    BS_nx = length(BS_id);
    % -- define a dummy for agreements
    BS_agree = zeros(BS_nx,1); BS_agree(BS_nx) = 1; 
    BS_next_States = zeros(BS_nx,1); BS_next_States(BS_nx) = NaN;
    for i = 1:BS_nx-1,
        if BS_id(i) == BS_id(i+1), 
           BS_agree(i) = 0;
           BS_next_States(i) = BS_States(i+1);
        else BS_agree(i) = 1;
             BS_next_States(i) = NaN;
        end;
    end;

    tabout_t = crosstab(BS_States,BS_agree);
    if min(min(tabout_t,[],2)) == 0,
        continue;
    end;
    
    [c_t,b_t,diff_t,TM_t,Rej_t,SampRej_t,bdflag_t,b_bdflag_t,tabout_t] = ...
                               BS_EstCakeNew(BS_Cake,BS_States,BS_agree,BS_next_States,BS_id,1,Rej);
    BS_c_hat(:,t) = c_t; BS_b_hat(:,t) = b_t;  BS_diff_hat(:,t) = diff_t;
    BS_TM_hat(:,:,t) = TM_t; BS_flag_mat(:,:,t) = [Rej_t,SampRej_t,bdflag_t,b_bdflag_t];
    t = t + 1;
end;
toc;
% report the interval estimates
disp(sprintf('Confidence Intervals using "episode" Bootstrap : (Bootstrap Iterations : %g)',T));
disp('95% C.I. for cakes : ');
disp(prctile(BS_c_hat',[5 95])');
disp('95% C.I. for differences : ');
disp(prctile(BS_diff_hat',[5 95])');
disp('90% C.I. for discount factors : ');
disp(prctile(BS_b_hat',[5 95]));
% disp('Flags claculated from the BS data : sum of [Rej Sample_Rej Bound_Flag]')
% disp(sum(BS_flag_mat,3));
% % disp('Flags claculated from the BS data : sum of [Rej Sample_Rej]')
% % disp(sum(BS_flag_mat,3));
disp('****************************************************************');

